package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

package .tmp;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function4;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.attributeslib.AttributesLib;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import dev.shadowsoffire.placebo.util.StepFunction;
import java.util.Map;
import java.util.UUID;
import java.util.function.BiConsumer;
import net.minecraft.network.chat.Component;
import net.minecraft.world.entity.ai.attributes.Attribute;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.registries.ForgeRegistries;

public class AttributeBonus extends GemBonus {
  public static Codec<dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.AttributeBonus> CODEC;
  
  protected final Attribute attribute;
  
  protected final AttributeModifier.Operation operation;
  
  protected final Map<LootRarity, StepFunction> values;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)ForgeRegistries.ATTRIBUTES.getCodec().fieldOf("attribute").forGetter(()), (App)PlaceboCodecs.enumCodec(AttributeModifier.Operation.class).fieldOf("operation").forGetter(()), (App)VALUES_CODEC.fieldOf("values").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.AttributeBonus::new));
  }
  
  public AttributeBonus(GemClass gemClass, Attribute attr, AttributeModifier.Operation op, Map<LootRarity, StepFunction> values) {
    super(Apotheosis.loc("attribute"), gemClass);
    this.attribute = attr;
    this.operation = op;
    this.values = values;
  }
  
  public void addModifiers(ItemStack gem, LootRarity rarity, BiConsumer<Attribute, AttributeModifier> map) {
    map.accept(this.attribute, read(gem, rarity, GemItem.getUUIDs(gem).get(0)));
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    return (Component)IFormattableAttribute.toComponent(this.attribute, read(gem, rarity, UUID.randomUUID()), AttributesLib.getTooltipFlag());
  }
  
  public dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.AttributeBonus validate() {
    Preconditions.checkNotNull(this.attribute, "Invalid AttributeBonus with null attribute");
    Preconditions.checkNotNull(this.operation, "Invalid AttributeBonus with null operation");
    Preconditions.checkNotNull(this.values, "Invalid AttributeBonus with null values");
    return this;
  }
  
  public boolean supports(LootRarity rarity) {
    return this.values.containsKey(rarity);
  }
  
  public int getNumberOfUUIDs() {
    return 1;
  }
  
  public AttributeModifier read(ItemStack gem, LootRarity rarity, UUID id) {
    return new AttributeModifier(id, "apoth.gem_modifier", ((StepFunction)this.values.get(rarity)).get(0.0F), this.operation);
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
}
